@@ -270,14 +270,14 @@ g_desktop_app_info_new_from_keyfile (GKeyFile *key_file)
         {
           char *p;
 
-          /* Work around a common mistake in desktop files */
+          /* Work around a common mistake in desktop files */    
           if ((p = strrchr (info->icon_name, '.')) != NULL &&
               (strcmp (p, ".png") == 0 ||
                strcmp (p, ".xpm") == 0 ||
-               strcmp (p, ".svg") == 0))
+               strcmp (p, ".svg") == 0)) 
             *p = 0;
 
-          info->icon = g_themed_icon_new (info->icon_name);
+	  info->icon = g_themed_icon_new (info->icon_name);
         }
     }
   
@@ -479,6 +479,14 @@ g_desktop_app_info_get_executable (GAppInfo *appinfo)
   return info->binary;
 }
 
+static const char *
+g_desktop_app_info_get_commandline (GAppInfo *appinfo)
+{
+  GDesktopAppInfo *info = G_DESKTOP_APP_INFO (appinfo);
+  
+  return info->exec;
+}
+
 static GIcon *
 g_desktop_app_info_get_icon (GAppInfo *appinfo)
 {
@@ -542,9 +550,32 @@ expand_macro (char              macro,
 {
   GList *uris = *uri_list;
   char *expanded;
-  
+  gboolean force_file_uri;
+  char force_file_uri_macro;
+
   g_return_if_fail (exec != NULL);
-  
+
+  /* On %u and %U, pass POSIX file path pointing to the URI via
+   * the FUSE mount in ~/.gvfs. Note that if the FUSE daemon isn't
+   * running or the URI doesn't have a POSIX file path via FUSE
+   * we'll just pass the URI.
+   */
+  switch (macro)
+    {
+    case 'u':
+      force_file_uri_macro = 'f';
+      force_file_uri = TRUE;
+      break;
+    case 'U':
+      force_file_uri_macro = 'F';
+      force_file_uri = TRUE;
+      break;
+    default:
+      force_file_uri_macro = macro;
+      force_file_uri = FALSE;
+      break;
+    }
+
   switch (macro)
     {
     case 'u':
@@ -553,7 +584,17 @@ expand_macro (char              macro,
     case 'n':
       if (uris)
 	{
-	  expanded = expand_macro_single (macro, uris->data);
+          if (!force_file_uri)
+            {
+              expanded = expand_macro_single (macro, uris->data);
+            }
+          else
+            {
+              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              if (expanded == NULL)
+                expanded = expand_macro_single (macro, uris->data);
+            }
+
 	  if (expanded)
 	    {
 	      g_string_append (exec, expanded);
@@ -570,7 +611,17 @@ expand_macro (char              macro,
     case 'N':
       while (uris)
 	{
-	  expanded = expand_macro_single (macro, uris->data);
+          if (!force_file_uri)
+            {
+              expanded = expand_macro_single (macro, uris->data);
+            }
+          else
+            {
+              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              if (expanded == NULL)
+                expanded = expand_macro_single (macro, uris->data);
+            }
+
 	  if (expanded)
 	    {
 	      g_string_append (exec, expanded);
@@ -1183,7 +1234,8 @@ update_mimeapps_list (const char  *desktop_id,
   GList *system_list, *l;
   gsize length, data_size;
   char *data;
-  int i, j;
+  int i, j, k;
+  char **content_types;
 
   /* Don't add both at start and end */
   g_assert (!(add_as_default && add_non_default));
@@ -1203,105 +1255,139 @@ update_mimeapps_list (const char  *desktop_id,
       key_file = g_key_file_new ();
     }
 
-  /* Add to the right place in the list */
-  
-  length = 0;
-  old_list = g_key_file_get_string_list (key_file, ADDED_ASSOCIATIONS_GROUP,
-					 content_type, &length, NULL);
-
-  list = g_new (char *, 1 + length + 1);
-
-  i = 0;
-  if (add_as_default)
-    list[i++] = g_strdup (desktop_id);
-  if (old_list)
+  if (content_type)
     {
-      for (j = 0; old_list[j] != NULL; j++)
-	{
-	  if (strcmp (old_list[j], desktop_id) != 0)
-	    list[i++] = g_strdup (old_list[j]);
-	  else if (add_non_default)
-	    {
-	      /* If adding as non-default, and its already in,
-		 don't change order of desktop ids */
-	      add_non_default = FALSE;
-	      list[i++] = g_strdup (old_list[j]);
-	    }
-	}
+      content_types = g_new (char *, 2);
+      content_types[0] = g_strdup (content_type);
+      content_types[1] = NULL;
     }
-       
-  if (add_non_default)
-    {
-      /* We're adding as non-default, and it wasn't already in the list,
-	 so we add at the end. But to avoid listing the app before the
-	 current system default (thus changing the default) we have to
-	 add the current list of (not yet listed) apps before it. */
-      
-      list[i] = NULL; /* Terminate current list so we can use it */
- 	  system_list =  get_all_desktop_entries_for_mime_type (content_type, list);
-	  
- 	  list = g_renew (char *, list, 1 + length + g_list_length (system_list) + 1);
- 	  
- 	  for (l = system_list; l != NULL; l = l->next)
- 	    {
- 	      list[i++] = l->data; /* no strdup, taking ownership */
- 	      if (strcmp (l->data, desktop_id) == 0)
- 		add_non_default = FALSE;
- 	    }
- 	  g_list_free (system_list);
-	  
- 	  if (add_non_default)
- 	    list[i++] = g_strdup (desktop_id);
-    }
-  
-  list[i] = NULL;
-  
-  g_strfreev (old_list);
-
-  g_key_file_set_string_list (key_file,
-			      ADDED_ASSOCIATIONS_GROUP,
-			      content_type,
-			      (const char * const *)list, i);
-
-  g_strfreev (list);
-
-  /* Remove from removed associations group (unless remove) */
-  
-  length = 0;
-  old_list = g_key_file_get_string_list (key_file, REMOVED_ASSOCIATIONS_GROUP,
-					 content_type, &length, NULL);
-
-  list = g_new (char *, 1 + length + 1);
-
-  i = 0;
-  if (remove)
-    list[i++] = g_strdup (desktop_id);
-  if (old_list)
-    {
-      for (j = 0; old_list[j] != NULL; j++)
-	{
-	  if (strcmp (old_list[j], desktop_id) != 0)
-	    list[i++] = g_strdup (old_list[j]);
-	}
-    }
-  list[i] = NULL;
-  
-  g_strfreev (old_list);
-
-  if (list[0] == NULL)
-    g_key_file_remove_key (key_file,
-			   REMOVED_ASSOCIATIONS_GROUP,
-			   content_type,
-			   NULL);
   else
-    g_key_file_set_string_list (key_file,
-				REMOVED_ASSOCIATIONS_GROUP,
-				content_type,
-				(const char * const *)list, i);
-
-  g_strfreev (list);
+    {
+      content_types = g_key_file_get_keys (key_file, ADDED_ASSOCIATIONS_GROUP, NULL, NULL);
+    }
 
+  for (k = 0; content_types && content_types[k]; k++)
+    { 
+      /* Add to the right place in the list */
   
+      length = 0;
+      old_list = g_key_file_get_string_list (key_file, ADDED_ASSOCIATIONS_GROUP,
+					     content_types[k], &length, NULL);
+
+      list = g_new (char *, 1 + length + 1);
+
+      i = 0;
+      if (add_as_default)
+        list[i++] = g_strdup (desktop_id);
+      if (old_list)
+        {
+          for (j = 0; old_list[j] != NULL; j++)
+	    {
+	      if (g_strcmp0 (old_list[j], desktop_id) != 0)
+	        list[i++] = g_strdup (old_list[j]);
+	      else if (add_non_default)
+		{
+		  /* If adding as non-default, and its already in,
+		     don't change order of desktop ids */
+		  add_non_default = FALSE;
+		  list[i++] = g_strdup (old_list[j]);
+		}
+	    }
+        }
+      
+      if (add_non_default)
+	{
+	  /* We're adding as non-default, and it wasn't already in the list,
+	     so we add at the end. But to avoid listing the app before the
+	     current system default (thus changing the default) we have to
+	     add the current list of (not yet listed) apps before it. */
+
+	  list[i] = NULL; /* Terminate current list so we can use it */
+	  system_list =  get_all_desktop_entries_for_mime_type (content_type, list);
+	  
+	  list = g_renew (char *, list, 1 + length + g_list_length (system_list) + 1);
+	  
+	  for (l = system_list; l != NULL; l = l->next)
+	    {
+	      list[i++] = l->data; /* no strdup, taking ownership */
+	      if (g_strcmp0 (l->data, desktop_id) == 0)
+		add_non_default = FALSE;
+	    }
+	  g_list_free (system_list);
+		  
+	  if (add_non_default)
+	    list[i++] = g_strdup (desktop_id);
+	}
+      
+      list[i] = NULL;
+  
+      g_strfreev (old_list);
+
+      if (list[0] == NULL || desktop_id == NULL)
+        g_key_file_remove_key (key_file,
+			       ADDED_ASSOCIATIONS_GROUP,
+			       content_types[k],
+			       NULL);
+      else
+        g_key_file_set_string_list (key_file,
+			            ADDED_ASSOCIATIONS_GROUP,
+			            content_types[k],
+			            (const char * const *)list, i);
+   
+      g_strfreev (list);
+    }
+  
+  if (content_type)
+    {
+      /* reuse the list from above */
+    }
+  else
+    {
+      g_strfreev (content_types);
+      content_types = g_key_file_get_keys (key_file, REMOVED_ASSOCIATIONS_GROUP, NULL, NULL);
+    }
+
+  for (k = 0; content_types && content_types[k]; k++) 
+    {
+      /* Remove from removed associations group (unless remove) */
+  
+      length = 0;
+      old_list = g_key_file_get_string_list (key_file, REMOVED_ASSOCIATIONS_GROUP,
+					     content_types[k], &length, NULL);
+
+      list = g_new (char *, 1 + length + 1);
+
+      i = 0;
+      if (remove)
+        list[i++] = g_strdup (desktop_id);
+      if (old_list)
+        {
+          for (j = 0; old_list[j] != NULL; j++)
+	    {
+	      if (g_strcmp0 (old_list[j], desktop_id) != 0)
+	        list[i++] = g_strdup (old_list[j]);
+	    }
+        }
+      list[i] = NULL;
+  
+      g_strfreev (old_list);
+
+      if (list[0] == NULL || desktop_id == NULL)
+        g_key_file_remove_key (key_file,
+			       REMOVED_ASSOCIATIONS_GROUP,
+			       content_types[k],
+			       NULL);
+      else
+        g_key_file_set_string_list (key_file,
+				    REMOVED_ASSOCIATIONS_GROUP,
+				    content_types[k],
+				    (const char * const *)list, i);
+
+      g_strfreev (list);
+    }
+  
+  g_strfreev (content_types);  
+
   data = g_key_file_to_data (key_file, &data_size, error);
   g_key_file_free (key_file);
   
@@ -1465,8 +1551,8 @@ g_desktop_app_info_remove_supports_type (GAppInfo    *appinfo,
 }
 
 static gboolean
-g_desktop_app_info_ensure_saved (GDesktopAppInfo *info,
-				 GError **error)
+g_desktop_app_info_ensure_saved (GDesktopAppInfo  *info,
+				 GError          **error)
 {
   GKeyFile *key_file;
   char *dirname;
@@ -1555,6 +1641,43 @@ g_desktop_app_info_ensure_saved (GDesktopAppInfo *info,
   return TRUE;
 }
 
+static gboolean
+g_desktop_app_info_can_delete (GAppInfo *appinfo)
+{
+  GDesktopAppInfo *info = G_DESKTOP_APP_INFO (appinfo);
+
+  if (info->filename)
+    {
+      if (strstr (info->filename, "/userapp-"))
+        return g_access (info->filename, W_OK) == 0;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+g_desktop_app_info_delete (GAppInfo *appinfo)
+{
+  GDesktopAppInfo *info = G_DESKTOP_APP_INFO (appinfo);
+  
+  if (info->filename)
+    { 
+      if (g_remove (info->filename) == 0)
+        {
+          update_mimeapps_list (info->desktop_id, NULL, FALSE, FALSE, FALSE, NULL);
+
+          g_free (info->filename);
+          info->filename = NULL;
+          g_free (info->desktop_id);
+          info->desktop_id = NULL;
+
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 /**
  * g_app_info_create_from_commandline:
  * @commandline: the commandline to use
@@ -1628,6 +1751,9 @@ g_desktop_app_info_iface_init (GAppInfoIface *iface)
   iface->add_supports_type = g_desktop_app_info_add_supports_type;
   iface->can_remove_supports_type = g_desktop_app_info_can_remove_supports_type;
   iface->remove_supports_type = g_desktop_app_info_remove_supports_type;
+  iface->can_delete = g_desktop_app_info_can_delete;
+  iface->do_delete = g_desktop_app_info_delete;
+  iface->get_commandline = g_desktop_app_info_get_commandline;
 }
 
 static gboolean
@@ -1685,6 +1811,22 @@ g_app_info_get_all_for_type (const char *content_type)
   return g_list_reverse (infos);
 }
 
+/**
+ * g_app_info_reset_type_associations:
+ * @content_type: a content type 
+ *
+ * Removes all changes to the type associations done by
+ * g_app_info_set_as_default_for_type(), 
+ * g_app_info_set_as_default_for_extension(), 
+ * g_app_info_add_supports_type() of g_app_info_remove_supports_type().
+ *
+ * Since: 2.20
+ */
+void
+g_app_info_reset_type_associations (const char *content_type)
+{
+  update_mimeapps_list (NULL, content_type, FALSE, FALSE, FALSE, NULL);
+}
 
 /**
  * g_app_info_get_default_for_type:
@@ -2521,7 +2663,7 @@ get_all_desktop_entries_for_mime_type (const char *base_mime_type,
       g_strfreev (anc);
     }
   g_ptr_array_add (array, NULL);
-  mime_types = g_ptr_array_free (array, FALSE);
+  mime_types = (char **)g_ptr_array_free (array, FALSE);
 
   G_LOCK (mime_info_cache);
   
